from setuptools import setup

setup(
    name="nODL",
    version="0.0.22",
    description="nODL (neastly Objective Database Library) - python library for creating and managing sqlite3 databases with object mapping.",
    author="nesquikcode",
    author_email="nesquik@nishine.ru",
    packages=["nODL"],
    install_requires=[
        "pydantic"
    ],
)